save_forecast <- function(data_cube,
			  recipe,
                          outdir,
                          agg="global") {
  # Loops over the years in the s2dv_cube containing a hindcast or forecast
  # and exports each year to a netCDF file.
  # data_cube: s2dv_cube containing the data and metadata
  # recipe: the auto-s2s recipe
  # outdir: directory where the files should be saved
  # agg: aggregation, "global" or "country"

  lalo <- c('longitude', 'latitude')
  
  variable <- data_cube$Variable$varName
  var.longname <- attr(data_cube$Variable, 'variable')$long_name
  global_attributes <- get_global_attributes(recipe)
  fcst.horizon <- tolower(recipe$Analysis$Horizon)
  store.freq <- recipe$Analysis$Variables$freq
  
  # Generate vector containing leadtimes
  ## TODO: Move to a separate function?
#-------------NEW-------------------
#data_cube$Dates$start[1]
#[1] "1993-11-01 18:00:00 UTC"

#NOTE: as.Date is not needed
#  dates <- as.Date(data_cube$Dates$start[1, ])
  if (store.freq == 'monthly_mean') {
    dates <- ClimProjDiags::Subset(as.Date(data_cube$Dates$start), 'syear', 1)
  } else {
    dates <- ClimProjDiags::Subset(as.Date(data_cube$Dates$start), 'file_date', 1)
  }
#NOTE: If time values overlap, sorting will mess up
#  dates <- sort(as.Date(data$hcst$Dates$start))
#  n_steps <- dim(data$hcst$data)['time'][[1]] # number of time steps
#  dates <- dates[1:n_steps]
  init_date <- as.Date(paste0(recipe$Analysis$Time$hcst_start,
		              recipe$Analysis$Time$sdate),
     	               format = '%Y%m%d')
  # Get time difference in months
  leadtimes <- interval(init_date, dates) %/% hours(1)
#---------------NEW_END-------------------

  syears <- seq(1:dim(data_cube$data)['syear'][[1]])
  for (i in syears) {
    # Select year from array and rearrange dimensions
    fcst <- data_cube$data[ , , , , i, , , , ] # Select year

    if (!("time" %in% names(dim(fcst)))) {
      dim(fcst) <- c("time" = 1, dim(fcst))
    }
    if (tolower(agg) == "global") {
      fcst <- list(Reorder(fcst, c(lalo, 'ensemble', 'time')))
    } else {
      fcst <- list(Reorder(fcst, c('country', 'ensemble', 'time')))
    }

    # Add metadata
    if (tolower(agg) == "country") {
      dims <- c('Country', 'time')
      var.expname <- paste0(variable, '_country') 
      var.sdname <- paste0("Country-Aggregated ", var.longname)
      var.units <- attr(data_cube$Variable, 'variable')$units
    } else {
      dims <- c(lalo, 'ensemble', 'time')
      var.expname <- variable
      var.sdname <- var.longname
      var.units <- attr(data_cube$Variable, 'variable')$units
    }
  
    metadata <- list(fcst = list(name = var.expname,
      			         standard_name = var.sdname,
      			         long_name = var.longname,
                                 units = var.units))
    attr(fcst[[1]], 'variables') <- metadata
    names(dim(fcst[[1]])) <- dims
    # Add global attributes
    global_attributes <- get_global_attributes(recipe)
    attr(fcst[[1]], 'global_attrs') <- global_attributes
    
    # Select start date
    fcst.sdate <- data_cube$load_parameters$dat1$file_date[[1]][i]

    # Get time dimension values and metadata
    times <- get_times(store.freq, fcst.horizon, leadtimes, fcst.sdate)
    time <- times$time
    time_step <- times$time_step

    # Generate name of output file
    outfile <- get_filename(outdir, data_cube$Variable$varName,
  			    fcst.sdate, fcst.sdate,
  			    agg, fcst.horizon, "exp")
  
    # Get grid data and metadata and export to netCDF
    if (tolower(agg) == "country") { 
      country <- get_countries(grid)
      ArrayToNc(append(country, time, fcst, time_step), outfile)
    } else {
      latitude <- data_cube$lat[1:length(data_cube$lat)]
      longitude <- data_cube$lon[1:length(data_cube$lon)]
      latlon <- get_latlon(latitude, longitude)
      # Compile variables into a list and export to netCDF
      vars <- list(latlon$lat, latlon$lon, time)
      vars <- c(vars, fcst, list(time_step))
      ArrayToNc(vars, outfile)
    }
  }
  print("##### FCST SAVED TO NETCDF FILE #####")
}



