#---------0. Set up environment
## Load packages
library(sf)
library(ggplot2)
library(rnaturalearth)
library(RColorBrewer)
library(cowplot) # to add s2dv legend
library(startR)
library(s2dv)

## Source function
#URL: https://earth.bsc.es/gitlab/es/s2dv/-/raw/develop-PlotRobinson/R/PlotRobinson.R
source("/esarchive/scratch/aho/tmp/PlotRobinson.R")
## Source unexported s2dv functions used in PlotRobinson
.KnownLonNames <- s2dv:::.KnownLonNames
.KnownLatNames <- s2dv:::.KnownLatNames
.IsColor <- s2dv:::.IsColor

## Projection source: https://epsg.io/

fileout_path <- "/esarchive/scratch/aho/tmp/"

#=====================================================================
# Global
#=====================================================================

#---------1. Load data
repos <- "/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc"
data <- Start(dat = list(list(name = 'system5c3s', path = repos)),
              var = 'tas',
              sdate = '20170101',
              ensemble = indices(1),
              time = indices(1),
              lon = 'all', 
              lon_reorder = CircularSort(-180, 180),
              lat = 'all',
              lat_reorder = Sort(decreasing = T),
              synonims = list(lat = c('lat', 'latitude'),
                              lon = c('lon', 'longitude')),
              return_vars = list(time = 'sdate', longitude = 'dat', latitude = 'dat'),
              retrieve = T)
lon <- attributes(data)$Variable$system5c3s$lon
lat <- attributes(data)$Variable$system5c3s$lat
data <- data - 273.15 # K to degC

#---------2. Create mask
mask <- drop(data)
mask[which(mask > -20 & mask < -10)] <- 0
mask[which(mask != 0)] <- 1

#---------3. Create colorbar breaks
## have exceeded values
brks1 <- seq(-40, 40, 10)
cols1 <- brewer.pal(9, 'PuOr')[-9]
## cover all values
#brks1 <- seq(-ceiling(max(abs(range(data)))), ceiling(max(abs(range(data)))), length.out = 11)

#---------4. Some projection options
tar_proj <- c(Robinson = "+proj=robin", #"ESRI:54030",
              Mollweide = "+proj=moll") #"ESRI:54009")


#---------5. Plot
## legend = 'ggplot2'
PlotRobinson(data, lon, lat, brks = brks1, cols = NULL, legend = 'ggplot2',
             target_proj = tar_proj['Robinson'], style = 'point',
             toptitle = "system5c3s monthly tas Jan 2017", caption = "Projection: Robinson",
             dots = mask, units = 'degC', colNA = 'red',
             width = 10, height = 6, fileout = paste0(fileout_path, '/PlotRobinson_ggplot.png'))

## legend = 's2dv'
PlotRobinson(data, lon, lat, brks = brks1, cols = cols1, legend = 's2dv',
             target_proj = tar_proj['Robinson'], style = 'point',
             toptitle = "system5c3s monthly tas Jan 2017", caption = "Projection: Robinson",
             dots = mask, units = "degC", col_inf = 'brown',
             width = 10, height = 5, 
             bar_extra_margin = c(0, 5, 0, 5),
             vertical = F, fileout = paste0(fileout_path, '/PlotRobinson_s2dv.png'))


#=====================================================================
# Region: Europe
#=====================================================================

#---------1. Crop EU region from global data
data_eu <- ClimProjDiags::Subset(data, c('lat', 'lon'), list(21:61, 161:221))
lon_eu <- lon[161:221]
lat_eu <- lat[21:61]

#---------2. Create mask
mask_eu <- ClimProjDiags::Subset(mask, c('lat', 'lon'), list(21:61, 161:221))

#---------3. Create colorbar breaks
## have exceeded values
brks_eu <- seq(-18, 18, length.out = 11)
## cover all values
#brks_eu <- seq(-ceiling(max(abs(range(data_eu)))), ceiling(max(abs(range(data_eu)))), length.out = 11)

#---------4. Some projection options
tar_proj_eu <- c("Lambert Europe" = "+proj=lcc +lat_0=30 +lon_0=10 +lat_1=43 +lat_2=62 +x_0=0 +y_0=0 +ellps=intl +units=m +no_defs +type=crs") #"ESRI:102014"

#---------5. Plot
## style = point; use 'mask'
PlotRobinson(data_eu, lon_eu, lat_eu, brks = brks_eu, cols = NULL, legend = 's2dv',
             target_proj = tar_proj_eu['Lambert Europe'], style = 'point',
             toptitle = "system5c3s monthly tas Jan 2017", 
             caption = paste0("Projection: Lambert Europe\nESRI:102014\n", "Europe"),
             mask = mask_eu, units = "degC",
             width = 8, height = 6, data_size = 1.3,
             crop_coastlines = c(lonmin = -20, lonmax = 40, latmin = 27, latmax = 70),
             triangle_ends = c(T, T), bar_extra_margin = c(0.4, 4, 0.4, 4),
             vertical = F, fileout = paste0(fileout_path, '/PlotRobinson_eu_pt.png'))

## style = polygon
PlotRobinson(data_eu, lon_eu, lat_eu, brks = brks_eu, cols = NULL, legend = 's2dv',
             target_proj = tar_proj_eu['Lambert Europe'], style = 'polygon',
             toptitle = "system5c3s monthly tas Jan 2017",
             caption = paste0("Projection: Lambert Europe\nESRI:102014\n", "Europe"),
             mask = mask_eu, units = "degC",
             width = 8, height = 6,
             crop_coastlines = c(lonmin = -20, lonmax = 40, latmin = 27, latmax = 70),
             triangle_ends = c(T, T), bar_extra_margin = c(0.4, 4, 0.4, 4),
             vertical = F, fileout = paste0(fileout_path, '/PlotRobinson_eu_poly.png'))


#=====================================================================
# Region: North Pole
#=====================================================================

#---------1. Crop EU region from global data
data_np <- ClimProjDiags::Subset(data, c('lat'), list(1:31))
lon_np <- lon
lat_np <- lat[1:31]

#---------2. Create mask
mask_np <- ClimProjDiags::Subset(mask, c('lat'), list(1:31))

#---------3. Create colorbar breaks
## have exceeded values
brks_np <- seq(-40, 0, length.out = 11)
## cover all values
#brks_np <- seq(-ceiling(max(abs(range(data_np)))), ceiling(max(abs(range(data_np)))), length.out = 11)

#---------4. Some projection options
tar_proj_np <- c(Arctic = "+proj=stere")#"EPSG:3995")

#---------5. Plot
#NOTE: A bit slow, be patience.
PlotRobinson(data_np, lon_np, lat_np, brks = brks_np, cols = NULL, legend = 's2dv',
             target_proj = tar_proj_np['Arctic'], style = 'polygon',
             toptitle = "system5c3s monthly tas Jan 2017",
             caption = paste0("Projection: WGS 84 / Arctic Polar Stereographic\nEPSG:3995\n", "North Pole"),
             dots = mask_np, dots_size = 1.5,
             units = "degC", color_fun = clim.palette("orangepurple"), 
             width = 8, height = 8, #data_size = 1.6,
             triangle_ends = c(T, T), bar_extra_margin = c(0.4, 4, 0.4, 4),
             vertical = F, fileout = paste0(fileout_path, '/PlotRobinson_np.png'))

# Compare with PlotStereoMap
#NOTE: Save the figure instead of plotting in pop-up window because it is slow.
PlotStereoMap(data_np, lon = lon_np, lat = lat_np, brks = brks_np, cols = NULL,
              color_fun = clim.palette("orangepurple"), 
              toptitle = "system5c3s monthly tas Jan 2017 (PlotStereoMap)", 
              width = 8, height = 8, filled.continent = F,
              dots = mask_np, units = "degC", title_scale = 0.8,
              fileout = paste0(fileout_path, '/PlotStereoMap_np.png'))

