#=========================================================
# This example shows how to combine PlotStereoMap() figures together in one file,
# if you want to have separate color bars for each plot.
# The trick is to define layout by layout() first, plot figures without color bar,
# and plot color bars by ColorBar().
# Directly plotting the figures with color bar doesn't work properly since PlotStereoMap() 
# uses layout() inside if color bar is requested. Which mean, inside PlotStereoMap(),
# the layout will be redefined.
#=========================================================

library(startR)
library(s2dv)

#-----------------------------------------
# Load example data
#-----------------------------------------
repos <- "/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$sdate$.nc"
data <- Start(dat = list(list(name = 'system5c3s', path = repos)),
              var = 'tas',
              sdate = '20170101',
              ensemble = indices(1),
              time = indices(1),
              lon = 'all',
              lon_reorder = CircularSort(-180, 180),
              lat = values(list(90, 60)), #'all',
              lat_reorder = Sort(decreasing = T),
              synonims = list(lat = c('lat', 'latitude'),
                              lon = c('lon', 'longitude')),
              return_vars = list(time = 'sdate', longitude = 'dat', latitude = 'dat'),
              retrieve = T)
lon <- attributes(data)$Variable$system5c3s$lon
lat <- attributes(data)$Variable$system5c3s$lat
data <- data - 273.15 # K to degC

## Create mask
mask <- drop(data)
mask[which(mask > -20 & mask < -10)] <- T
mask[which(mask != 1)] <- F

#----------------------------
# Save data as .rds file
#saveRDS(list(data = data, lon = lon, lat = lat, mask = mask), '~/tmp/example_data_PlotStereoMap.rds')
#tmp <- readRDS("~/tmp/example_data_PlotStereoMap.rds")
#data <- tmp$data
#lon <- tmp$lon
#lat <- tmp$lat
#mask <- tmp$mask
#----------------------------

brks <- seq(-40, 0, length.out = 11)

#----------------------------
# Plot single plot
#----------------------------
PlotStereoMap(data, lon = lon, lat = lat, brks = brks, cols = NULL,
              dots = mask,
              color_fun = clim.palette("orangepurple"),
              toptitle = "system5c3s monthly tas Jan 2017 (PlotStereoMap)",
              width = 8, height = 8, filled.continent = F,
              units = "degC", title_scale = 0.8,
              fileout = './tmp/PlotStereoMap_np.png')

#----------------------------
# Use PlotLayout()
#----------------------------
data2 <- list(drop(data), drop(data))

PlotLayout(PlotStereoMap, c('lat', 'lon'), var = data2, lon = lon, lat = lat,
           subplot_titles_scale = 1.2, toptitle = "This is the common title", 
           titles = c('aa', 'bb'),
           fileout = './tmp/PlotStereoMap_PlotLayout.png')

#----------------------------
# Manually create layout
#----------------------------
png("./tmp/PlotStereoMap_layout.png", width = 22, height = 14, unit = 'cm', res = 300)

layout.matrix <- matrix(c(1, 2, 3, 4), nrow = 2, ncol = 2)

layout(mat = layout.matrix, heights = c(lcm(10), lcm(2)), 
       width = c(lcm(10), lcm(10)))
#layout.show(4)

# Fig 1
PlotStereoMap(data, lon = lon, lat = lat, brks = brks, cols = NULL,
              dots = mask,
              color_fun = clim.palette("orangepurple"),
              filled.continent = F,
              units = "degC", drawleg = F)

# ColorBar for Fig 1
ColorBar(brks = brks, cols = NULL, vertical = F, 
         var_limits = range(data, na.rm = T),
         color_fun = clim.palette("orangepurple"),
         extra_margin = c(0.5, 0, 0.5, 0))

# Fig 2
PlotStereoMap(data, lon = lon, lat = lat, brks = brks, cols = NULL,
              dots = mask,
              color_fun = clim.palette("orangepurple"),
              filled.continent = F,
              units = "degC", drawleg = F)

# ColorBar for Fig 2
ColorBar(brks = brks, cols = NULL, vertical = F, var_limits = range(data, na.rm = T),
         color_fun = clim.palette("orangepurple"), extra_margin = c(0.5, 0, 0.5, 0))

# Top title
mtext("This is the top title for all plots", side = 3, line = -2, outer = TRUE,
      cex = 2)

dev.off()

